function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var axios = _interopDefault(require('axios'));
var XMLParser = _interopDefault(require('react-xml-parser'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function useData(initialData) {
  return react.useReducer(function (prevState, nextState) {
    return _extends({}, prevState, nextState);
  }, initialData);
}

// A type of promise-like that resolves synchronously and supports only one observer
const _Pact = /*#__PURE__*/(function() {
	function _Pact() {}
	_Pact.prototype.then = function(onFulfilled, onRejected) {
		const result = new _Pact();
		const state = this.s;
		if (state) {
			const callback = state & 1 ? onFulfilled : onRejected;
			if (callback) {
				try {
					_settle(result, 1, callback(this.v));
				} catch (e) {
					_settle(result, 2, e);
				}
				return result;
			} else {
				return this;
			}
		}
		this.o = function(_this) {
			try {
				const value = _this.v;
				if (_this.s & 1) {
					_settle(result, 1, onFulfilled ? onFulfilled(value) : value);
				} else if (onRejected) {
					_settle(result, 1, onRejected(value));
				} else {
					_settle(result, 2, value);
				}
			} catch (e) {
				_settle(result, 2, e);
			}
		};
		return result;
	};
	return _Pact;
})();

// Settles a pact synchronously
function _settle(pact, state, value) {
	if (!pact.s) {
		if (value instanceof _Pact) {
			if (value.s) {
				if (state & 1) {
					state = value.s;
				}
				value = value.v;
			} else {
				value.o = _settle.bind(null, pact, state);
				return;
			}
		}
		if (value && value.then) {
			value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
			return;
		}
		pact.s = state;
		pact.v = value;
		const observer = pact.o;
		if (observer) {
			observer(pact);
		}
	}
}

function _isSettledPact(thenable) {
	return thenable instanceof _Pact && thenable.s & 1;
}

// Asynchronously iterate through an object that has a length property, passing the index as the first argument to the callback (even as the length property changes)
function _forTo(array, body, check) {
	var i = -1, pact, reject;
	function _cycle(result) {
		try {
			while (++i < array.length && (!check || !check())) {
				result = body(i);
				if (result && result.then) {
					if (_isSettledPact(result)) {
						result = result.v;
					} else {
						result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
						return;
					}
				}
			}
			if (pact) {
				_settle(pact, 1, result);
			} else {
				pact = result;
			}
		} catch (e) {
			_settle(pact || (pact = new _Pact()), 2, e);
		}
	}
	_cycle();
	return pact;
}

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously implement a generic for loop
function _for(test, update, body) {
	var stage;
	for (;;) {
		var shouldContinue = test();
		if (_isSettledPact(shouldContinue)) {
			shouldContinue = shouldContinue.v;
		}
		if (!shouldContinue) {
			return result;
		}
		if (shouldContinue.then) {
			stage = 0;
			break;
		}
		var result = body();
		if (result && result.then) {
			if (_isSettledPact(result)) {
				result = result.s;
			} else {
				stage = 1;
				break;
			}
		}
		if (update) {
			var updateValue = update();
			if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
				stage = 2;
				break;
			}
		}
	}
	var pact = new _Pact();
	var reject = _settle.bind(null, pact, 2);
	(stage === 0 ? shouldContinue.then(_resumeAfterTest) : stage === 1 ? result.then(_resumeAfterBody) : updateValue.then(_resumeAfterUpdate)).then(void 0, reject);
	return pact;
	function _resumeAfterBody(value) {
		result = value;
		do {
			if (update) {
				updateValue = update();
				if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
					updateValue.then(_resumeAfterUpdate).then(void 0, reject);
					return;
				}
			}
			shouldContinue = test();
			if (!shouldContinue || (_isSettledPact(shouldContinue) && !shouldContinue.v)) {
				_settle(pact, 1, result);
				return;
			}
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
				return;
			}
			result = body();
			if (_isSettledPact(result)) {
				result = result.v;
			}
		} while (!result || !result.then);
		result.then(_resumeAfterBody).then(void 0, reject);
	}
	function _resumeAfterTest(shouldContinue) {
		if (shouldContinue) {
			result = body();
			if (result && result.then) {
				result.then(_resumeAfterBody).then(void 0, reject);
			} else {
				_resumeAfterBody(result);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
	function _resumeAfterUpdate() {
		if (shouldContinue = test()) {
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
			} else {
				_resumeAfterTest(shouldContinue);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
}

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

var WS_PROTOCOL = "https://",
    WS_START_PORT = 41951,
    WS_END_PORT = 41953,
    WS_SVC_HOST = "127.0.0.1",
    WS_SVC_HOST_LEGACY = "localhost",
    WS_SVC_PATH = "DYMO/DLS/Printing",
    WS_ACTIONS = {
  status: "StatusConnected",
  getPrinters: "GetPrinters",
  openLabel: "OpenLabelFile",
  printLabel: "PrintLabel",
  printLabel2: "PrintLabel2",
  renderLabel: "RenderLabel",
  loadImage: "LoadImageAsPngBase64",
  getJobStatus: "GetJobStatus"
};

function store(storage, key, data, timeout) {
  if (timeout === void 0) {
    timeout = null;
  }

  var expiration = timeout && moment().add(timeout, "seconds").format();
  storage.setItem(key, JSON.stringify({
    data: data,
    expiration: expiration
  }));
}

function retrieve(storage, key, data) {
  if (data === void 0) {
    data = null;
  }

  try {
    var item = JSON.parse(storage.getItem(key)),
        expired = !!item.expiration && moment(item.expiration) < moment();

    if (expired) {
      storage.removeItem(key);
      return data;
    }

    return item.data || data;
  } catch (err) {
    return data;
  }
}

function localStore(key, data, timeout) {
  if (timeout === void 0) {
    timeout = null;
  }

  store(localStorage, key, data, timeout);
}
function localRetrieve(key, data) {
  if (data === void 0) {
    data = null;
  }

  return retrieve(localStorage, key, data);
}

var dymoRequestBuilder = function dymoRequestBuilder(_ref) {
  var _ref$wsProtocol = _ref.wsProtocol,
      wsProtocol = _ref$wsProtocol === void 0 ? WS_PROTOCOL : _ref$wsProtocol,
      _ref$wsPath = _ref.wsPath,
      wsPath = _ref$wsPath === void 0 ? WS_SVC_PATH : _ref$wsPath,
      wsAction = _ref.wsAction,
      method = _ref.method,
      cancelToken = _ref.cancelToken,
      _ref$axiosOtherParams = _ref.axiosOtherParams,
      axiosOtherParams = _ref$axiosOtherParams === void 0 ? {} : _ref$axiosOtherParams;

  try {
    var _temp4 = function _temp4() {
      var _localRetrieve = localRetrieve("dymo-ws-request-params"),
          activeHost = _localRetrieve.activeHost,
          activePort = _localRetrieve.activePort;

      var dymoAxiosInstance = axios.create();
      dymoAxiosInstance.interceptors.response.use(function (response) {
        return response;
      }, function (error) {
        try {
          var _error$response;

          if (axios.isCancel(error) || (error === null || error === void 0 ? void 0 : (_error$response = error.response) === null || _error$response === void 0 ? void 0 : _error$response.status) === 500) {
            return Promise.reject(error);
          }

          return Promise.resolve(storeDymoRequestParams()).then(function () {
            return localRetrieve("dymo-ws-request-params") ? _catch(function () {
              var _localRetrieve2 = localRetrieve("dymo-ws-request-params"),
                  activeHost = _localRetrieve2.activeHost,
                  activePort = _localRetrieve2.activePort;

              return Promise.resolve(axios.request(_extends({
                url: dymoUrlBuilder(wsProtocol, activeHost, activePort, wsPath, wsAction),
                method: method,
                cancelToken: cancelToken
              }, axiosOtherParams))).then(Promise.resolve);
            }, function (error) {
              return Promise.reject(error);
            }) : Promise.reject(error);
          });
        } catch (e) {
          return Promise.reject(e);
        }
      });
      return Promise.resolve(dymoAxiosInstance.request(_extends({
        url: dymoUrlBuilder(wsProtocol, activeHost, activePort, wsPath, wsAction),
        method: method,
        cancelToken: cancelToken
      }, axiosOtherParams)));
    };

    var _temp5 = function () {
      if (!localRetrieve("dymo-ws-request-params")) {
        return Promise.resolve(storeDymoRequestParams()).then(function () {});
      }
    }();

    return Promise.resolve(_temp5 && _temp5.then ? _temp5.then(_temp4) : _temp4(_temp5));
  } catch (e) {
    return Promise.reject(e);
  }
};

var storeDymoRequestParams = function storeDymoRequestParams() {
  try {
    var _loop1Interrupt2 = false;
    var activeHost, activePort;

    if (localRetrieve("dymo-ws-request-params")) {
      activeHost = localRetrieve("dymo-ws-request-params").activeHost;
      activePort = localRetrieve("dymo-ws-request-params").activePort;
      window.localStorage.removeItem("dymo-ws-request-params");
    }

    var hostList = [WS_SVC_HOST, WS_SVC_HOST_LEGACY];
    return Promise.resolve(_forTo(hostList, function (currentHostIndex) {
      var currentPort = WS_START_PORT;
      return _for(function () {
        return !_loop1Interrupt2 && currentPort <= WS_END_PORT;
      }, function () {
        return currentPort++;
      }, function () {
        if (hostList[currentHostIndex] === activeHost && currentPort === Number.parseInt(activePort)) {
          return;
        }

        var _temp = _catch(function () {
          return Promise.resolve(axios.get(dymoUrlBuilder(WS_PROTOCOL, hostList[currentHostIndex], currentPort, WS_SVC_PATH, "status"))).then(function (response) {
            var _response$config$url$ = response.config.url.split("/")[2].split(":"),
                successRequestHost = _response$config$url$[0],
                successRequestPort = _response$config$url$[1];

            localStore("dymo-ws-request-params", {
              activeHost: successRequestHost,
              activePort: successRequestPort
            });
            _loop1Interrupt2 = true;
          });
        }, function () {});

        if (_temp && _temp.then) return _temp.then(function () {});
      });
    }, function () {
      return _loop1Interrupt2;
    }));
  } catch (e) {
    return Promise.reject(e);
  }
};

function dymoUrlBuilder(wsProtocol, wsHost, wsPort, wsPath, wsAction) {
  return "" + wsProtocol + wsHost + ":" + wsPort + "/" + wsPath + "/" + WS_ACTIONS[wsAction];
}
function getDymoPrintersFromXml(xml, modelPrinter) {
  var xmlParse = new XMLParser().parseFromString(xml);
  var labelWritersPrinters = xmlParse.getElementsByTagName(modelPrinter);
  var printers = [];
  labelWritersPrinters.map(function (printer) {
    var printerDetails = {};
    printer.children.map(function (item) {
      printerDetails[item.name.charAt(0).toLowerCase() + item.name.slice(1)] = item.value;
    });
    printers.push(printerDetails);
  });
  return printers;
}
function printLabel(printerName, labelXml, labelSetXml) {
  return dymoRequestBuilder({
    method: "POST",
    wsAction: "printLabel",
    axiosOtherParams: {
      data: "printerName=" + encodeURIComponent(printerName) + "&printParamsXml=&labelXml=" + encodeURIComponent(labelXml) + "&labelSetXml=" + (labelSetXml || "")
    }
  });
}

var dymoUrlBuilder$1 = dymoUrlBuilder;
var dymoRequestBuilder$1 = dymoRequestBuilder;
var getDymoPrintersFromXml$1 = getDymoPrintersFromXml;
var printLabel$1 = printLabel;
function useDymoCheckService(port) {
  var _useState = react.useState("initial"),
      status = _useState[0],
      setStatus = _useState[1];

  var tokenSource = react.useRef();
  react.useEffect(function () {
    if (tokenSource.current) {
      tokenSource.current.cancel();
    }

    tokenSource.current = axios.CancelToken.source();
    setStatus("loading");
    dymoRequestBuilder$1({
      method: "GET",
      wsAction: "status",
      cancelToken: tokenSource.current.token
    }).then(function () {
      tokenSource.current = null;
      setStatus("success");
    })["catch"](function (error) {
      if (!axios.isCancel(error)) {
        setStatus("error");
      }
    });
    return function () {
      if (tokenSource.current) {
        tokenSource.current.cancel();
      }
    };
  }, [port]);
  return status;
}
function useDymoFetchPrinters(statusDymoService, modelPrinter, port) {
  if (modelPrinter === void 0) {
    modelPrinter = "LabelWriterPrinter";
  }

  var _useData = useData({
    statusFetchPrinters: "initial",
    printers: [],
    error: null
  }),
      data = _useData[0],
      setData = _useData[1];

  var tokenSource = react.useRef();
  react.useEffect(function () {
    if (statusDymoService === "success") {
      if (tokenSource.current) {
        tokenSource.current.cancel();
      }

      tokenSource.current = axios.CancelToken.source();
      setData({
        statusFetchPrinters: "loading"
      });
      dymoRequestBuilder$1({
        method: "GET",
        wsAction: "getPrinters",
        cancelToken: tokenSource.current.token
      }).then(function (response) {
        tokenSource.current = null;
        setData({
          statusFetchPrinters: "success",
          printers: getDymoPrintersFromXml$1(response.data, modelPrinter),
          error: null
        });
      })["catch"](function (error) {
        if (!axios.isCancel(error)) {
          setData({
            statusFetchPrinters: "error",
            printers: [],
            error: error
          });
        }
      });
    }

    return function () {
      if (tokenSource.current) {
        tokenSource.current.cancel();
      }
    };
  }, [modelPrinter, port, setData, statusDymoService]);
  return data;
}
function useDymoOpenLabel(statusDymoService, labelXML, port) {
  var _useData2 = useData({
    statusOpenLabel: "initial",
    label: null,
    error: null
  }),
      data = _useData2[0],
      setData = _useData2[1];

  var tokenSource = react.useRef();
  react.useEffect(function () {
    if (statusDymoService === "success") {
      if (tokenSource.current) {
        tokenSource.current.cancel();
      }

      tokenSource.current = axios.CancelToken.source();
      setData({
        statusOpenLabel: "loading"
      });
      dymoRequestBuilder$1({
        method: "POST",
        wsAction: "renderLabel",
        cancelToken: tokenSource.current.token,
        axiosOtherParams: {
          data: "labelXml=" + encodeURIComponent(labelXML) + "&renderParamsXml=&printerName="
        },
        headers: {
          "Access-Control-Request-Private-Network": true,
          "Access-Control-Allow-Origin": "*"
        }
      }).then(function (response) {
        tokenSource.current = null;
        setData({
          statusOpenLabel: "success",
          label: response.data,
          error: null
        });
      })["catch"](function (error) {
        if (!axios.isCancel(error)) {
          setData({
            statusOpenLabel: "error",
            label: null,
            error: error
          });
        }
      });
    }

    return function () {
      if (tokenSource.current) {
        tokenSource.current.cancel();
      }
    };
  }, [statusDymoService, labelXML, setData, port]);
  return data;
}

exports.dymoRequestBuilder = dymoRequestBuilder$1;
exports.dymoUrlBuilder = dymoUrlBuilder$1;
exports.getDymoPrintersFromXml = getDymoPrintersFromXml$1;
exports.printLabel = printLabel$1;
exports.useDymoCheckService = useDymoCheckService;
exports.useDymoFetchPrinters = useDymoFetchPrinters;
exports.useDymoOpenLabel = useDymoOpenLabel;
//# sourceMappingURL=index.js.map
